/**************************************************************************************
 * Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 * http://esper.codehaus.org                                                          *
 * http://www.espertech.com                                                           *
 * ---------------------------------------------------------------------------------- *
 * The software in this package is published under the terms of the GPL license       *
 * a copy of which has been included with this distribution in the license.txt file.  *
 **************************************************************************************/
package com.espertech.esper.epl.core;

import com.espertech.esper.client.EPException;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.EventType;
import com.espertech.esper.epl.db.PollExecStrategy;
import com.espertech.esper.event.EventAdapterService;
import net.sf.cglib.reflect.FastMethod;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

/**
 * Viewable providing historical data from a database.
 */
public abstract class MethodPollingExecStrategyBase implements PollExecStrategy
{
    private static final Log log = LogFactory.getLog(MethodPollingExecStrategyBase.class);

    protected final EventAdapterService eventAdapterService;
    protected final FastMethod method;
    protected final EventType eventType;

    public MethodPollingExecStrategyBase(EventAdapterService eventAdapterService, FastMethod method, EventType eventType) {
        this.eventAdapterService = eventAdapterService;
        this.method = method;
        this.eventType = eventType;
    }

    protected abstract List<EventBean> handleResult(Object invocationResult);

    protected boolean checkNonNullArrayValue(Object value) {
        if (value == null) {
            log.warn("Expected non-null return result from method '" + method.getName() + "', but received null array element value");
            return false;
        }
        return true;
    }

    public void start()
    {
    }

    public void done()
    {
    }

    public void destroy()
    {
    }

    public List<EventBean> poll(Object[] lookupValues)
    {
        List<EventBean> rowResult = null;
        try {
            Object invocationResult = method.invoke(null, lookupValues);
            if (invocationResult != null) {
                return handleResult(invocationResult);
            }
        }
        catch (InvocationTargetException ex) {
            throw new EPException("Method '" + method.getName() + "' of class '" + method.getJavaMethod().getDeclaringClass().getName() +
                    "' reported an exception: " + ex.getTargetException(), ex.getTargetException());
        }

        return rowResult;
    }
}
