/*
 * *************************************************************************************
 *  Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 *  http://esper.codehaus.org                                                          *
 *  http://www.espertech.com                                                           *
 *  ---------------------------------------------------------------------------------- *
 *  The software in this package is published under the terms of the GPL license       *
 *  a copy of which has been included with this distribution in the license.txt file.  *
 * *************************************************************************************
 */

package com.espertech.esper.epl.datetime.eval;

public class TimePeriod {
    private Integer years;
    private Integer months;
    private Integer weeks;
    private Integer days;
    private Integer hours;
    private Integer minutes;
    private Integer seconds;
    private Integer milliseconds;

    public TimePeriod(Integer years, Integer months, Integer weeks, Integer days, Integer hours, Integer minutes, Integer seconds, Integer milliseconds) {
        this.years = years;
        this.months = months;
        this.weeks = weeks;
        this.days = days;
        this.hours = hours;
        this.minutes = minutes;
        this.seconds = seconds;
        this.milliseconds = milliseconds;
    }

    public Integer getYears() {
        return years;
    }

    public Integer getMonths() {
        return months;
    }

    public Integer getWeeks() {
        return weeks;
    }

    public Integer getDays() {
        return days;
    }

    public Integer getHours() {
        return hours;
    }

    public Integer getMinutes() {
        return minutes;
    }

    public Integer getSeconds() {
        return seconds;
    }

    public Integer getMilliseconds() {
        return milliseconds;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        TimePeriod that = (TimePeriod) o;

        if (days != null ? !days.equals(that.days) : that.days != null) return false;
        if (hours != null ? !hours.equals(that.hours) : that.hours != null) return false;
        if (milliseconds != null ? !milliseconds.equals(that.milliseconds) : that.milliseconds != null) return false;
        if (minutes != null ? !minutes.equals(that.minutes) : that.minutes != null) return false;
        if (months != null ? !months.equals(that.months) : that.months != null) return false;
        if (seconds != null ? !seconds.equals(that.seconds) : that.seconds != null) return false;
        if (weeks != null ? !weeks.equals(that.weeks) : that.weeks != null) return false;
        if (years != null ? !years.equals(that.years) : that.years != null) return false;

        return true;
    }

    public int hashCode() {
        int result = years != null ? years.hashCode() : 0;
        result = 31 * result + (months != null ? months.hashCode() : 0);
        result = 31 * result + (weeks != null ? weeks.hashCode() : 0);
        result = 31 * result + (days != null ? days.hashCode() : 0);
        result = 31 * result + (hours != null ? hours.hashCode() : 0);
        result = 31 * result + (minutes != null ? minutes.hashCode() : 0);
        result = 31 * result + (seconds != null ? seconds.hashCode() : 0);
        result = 31 * result + (milliseconds != null ? milliseconds.hashCode() : 0);
        return result;
    }

    public boolean hasNegativeValues() {
        if (years != null && years < 0) {
            return true;
        }
        if (months != null && months < 0) {
            return true;
        }
        if (weeks != null && weeks < 0) {
            return true;
        }
        if (days != null && days < 0) {
            return true;
        }
        if (hours != null && hours < 0) {
            return true;
        }
        if (minutes != null && minutes < 0) {
            return true;
        }
        if (seconds != null && seconds < 0) {
            return true;
        }
        if (milliseconds != null && milliseconds < 0) {
            return true;
        }
        return false;
    }
}
