/**************************************************************************************
 * Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 * http://esper.codehaus.org                                                          *
 * http://www.espertech.com                                                           *
 * ---------------------------------------------------------------------------------- *
 * The software in this package is published under the terms of the GPL license       *
 * a copy of which has been included with this distribution in the license.txt file.  *
 **************************************************************************************/
package com.espertech.esper.epl.expression.methodagg;

import com.espertech.esper.epl.agg.access.AggregationAccessor;
import com.espertech.esper.epl.agg.access.AggregationAgent;
import com.espertech.esper.epl.agg.access.AggregationStateKey;
import com.espertech.esper.epl.agg.aggregator.AggregationMethod;
import com.espertech.esper.epl.agg.service.AggregationMethodFactory;
import com.espertech.esper.epl.agg.service.AggregationMethodFactoryUtil;
import com.espertech.esper.epl.agg.service.AggregationStateFactory;
import com.espertech.esper.epl.core.MethodResolutionService;
import com.espertech.esper.epl.expression.baseagg.ExprAggregateNodeBase;
import com.espertech.esper.epl.expression.core.ExprValidationException;

public class ExprCountEverNodeFactory implements AggregationMethodFactory
{
    private final ExprCountEverNode parent;
    private final boolean ignoreNulls;

    public ExprCountEverNodeFactory(ExprCountEverNode parent, boolean ignoreNulls) {
        this.parent = parent;
        this.ignoreNulls = ignoreNulls;
    }

    public boolean isAccessAggregation() {
        return false;
    }

    public Class getResultType()
    {
        return long.class;
    }

    public AggregationStateKey getAggregationStateKey(boolean isMatchRecognize) {
        throw new IllegalStateException("Not an access aggregation function");
    }

    public AggregationStateFactory getAggregationStateFactory(boolean isMatchRecognize) {
        throw new IllegalStateException("Not an access aggregation function");
    }

    public AggregationAccessor getAccessor() {
        throw new IllegalStateException("Not an access aggregation function");
    }

    public AggregationMethod make(MethodResolutionService methodResolutionService, int agentInstanceId, int groupId, int aggregationId) {
        return methodResolutionService.makeCountEverValueAggregator(agentInstanceId, groupId, aggregationId, parent.hasFilter(), ignoreNulls);
    }

    public ExprAggregateNodeBase getAggregationExpression() {
        return parent;
    }

    public void validateIntoTableCompatible(AggregationMethodFactory intoTableAgg) throws ExprValidationException {
        AggregationMethodFactoryUtil.validateAggregationType(this, intoTableAgg);
        ExprCountEverNodeFactory that = (ExprCountEverNodeFactory) intoTableAgg;
        if (that.ignoreNulls != ignoreNulls) {
            throw new ExprValidationException("The aggregation declares " +
                    (ignoreNulls ? "ignore-nulls" : "no-ignore-nulls") +
                    " and provided is " +
                    (that.ignoreNulls ? "ignore-nulls" : "no-ignore-nulls"));
        }
        AggregationMethodFactoryUtil.validateAggregationFilter(parent.hasFilter(), that.parent.hasFilter());
    }

    public AggregationAgent getAggregationStateAgent() {
        return null;
    }
}

