/**************************************************************************************
 * Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 * http://esper.codehaus.org                                                          *
 * http://www.espertech.com                                                           *
 * ---------------------------------------------------------------------------------- *
 * The software in this package is published under the terms of the GPL license       *
 * a copy of which has been included with this distribution in the license.txt file.  *
 **************************************************************************************/
package com.espertech.esper.epl.expression.methodagg;

import com.espertech.esper.epl.agg.service.AggregationMethodFactory;
import com.espertech.esper.epl.expression.baseagg.ExprAggregateNode;
import com.espertech.esper.epl.expression.baseagg.ExprAggregateNodeBase;
import com.espertech.esper.epl.expression.core.ExprValidationContext;
import com.espertech.esper.epl.expression.core.ExprValidationException;

/**
 * Represents the "firstever" aggregate function is an expression tree.
 */
public class ExprFirstEverNode extends ExprAggregateNodeBase
{
    private static final long serialVersionUID = 1436994080693454617L;

    /**
     * Ctor.
     * @param distinct - flag indicating unique or non-unique value aggregation
     */
    public ExprFirstEverNode(boolean distinct)
    {
        super(distinct);
    }

    public AggregationMethodFactory validateAggregationChild(ExprValidationContext validationContext) throws ExprValidationException
    {
        if (this.getChildNodes().length > 2)
        {
            throw new ExprValidationException("First aggregation node must have less then 2 child nodes");
        }
        if (this.getChildNodes().length == 2) {
            super.validateFilter(this.getChildNodes()[1].getExprEvaluator());
        }
        return new ExprFirstEverNodeFactory(this, this.getChildNodes()[0].getExprEvaluator().getType());
    }

    public boolean hasFilter() {
        return this.getChildNodes().length == 2;
    }

    public String getAggregationFunctionName()
    {
        return "firstever";
    }

    public final boolean equalsNodeAggregateMethodOnly(ExprAggregateNode node)
    {
        return node instanceof ExprFirstEverNode;
    }
}