/*
 * *************************************************************************************
 *  Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 *  http://esper.codehaus.org                                                          *
 *  http://www.espertech.com                                                           *
 *  ---------------------------------------------------------------------------------- *
 *  The software in this package is published under the terms of the GPL license       *
 *  a copy of which has been included with this distribution in the license.txt file.  *
 * *************************************************************************************
 */

package com.espertech.esper.epl.parse;

import com.espertech.esper.epl.generated.EsperEPL2GrammarParser;
import com.espertech.esper.epl.spec.MatchRecognizeSkipEnum;
import org.antlr.v4.runtime.CommonTokenStream;

/**
 * Helper class for walking the match-recognize AST.
 */
public class ASTMatchRecognizeHelper {

    private final static String message = "Match-recognize AFTER clause must be either AFTER MATCH SKIP TO LAST ROW or AFTER MATCH SKIP TO NEXT ROW or AFTER MATCH SKIP TO CURRENT ROW";

    public static MatchRecognizeSkipEnum parseSkip(CommonTokenStream tokenStream, EsperEPL2GrammarParser.MatchRecogMatchesAfterSkipContext ctx) {

        if ((!ctx.i1.getText().toUpperCase().equals("MATCH")) ||
            (!ctx.i2.getText().toUpperCase().equals("SKIP")) ||
            (!ctx.i5.getText().toUpperCase().equals("ROW"))
            ) {
            throw ASTWalkException.from(message, tokenStream, ctx);
        }

        if ((!ctx.i3.getText().toUpperCase().equals("TO")) &&
            (!ctx.i3.getText().toUpperCase().equals("PAST"))
            ) {
            throw ASTWalkException.from(message, tokenStream, ctx);
        }

        if (ctx.i4.getText().toUpperCase().equals("LAST")) {
            return MatchRecognizeSkipEnum.PAST_LAST_ROW;
        }
        else if (ctx.i4.getText().toUpperCase().equals("NEXT")) {
            return MatchRecognizeSkipEnum.TO_NEXT_ROW;
        }
        else if (ctx.i4.getText().toUpperCase().equals("CURRENT")) {
            return MatchRecognizeSkipEnum.TO_CURRENT_ROW;
        }
        throw ASTWalkException.from(message);
    }
}
