/*
 * *************************************************************************************
 *  Copyright (C) 2008 EsperTech, Inc. All rights reserved.                            *
 *  http://esper.codehaus.org                                                          *
 *  http://www.espertech.com                                                           *
 *  ---------------------------------------------------------------------------------- *
 *  The software in this package is published under the terms of the GPL license       *
 *  a copy of which has been included with this distribution in the license.txt file.  *
 * *************************************************************************************
 */

package com.espertech.esper.epl.expression;

import com.espertech.esper.epl.expression.core.ExprNodeUtility;
import com.espertech.esper.epl.expression.core.ExprValidationException;
import com.espertech.esper.epl.expression.core.ExprVariableNodeImpl;
import com.espertech.esper.epl.expression.ops.ExprInNode;
import com.espertech.esper.epl.variable.VariableService;
import com.espertech.esper.epl.variable.VariableServiceImpl;
import com.espertech.esper.support.epl.SupportExprNodeFactory;
import junit.framework.TestCase;

public class TestExprVariableNode extends TestCase
{
    private ExprVariableNodeImpl varNode;
    private VariableService variableService;

    public void setUp() throws Exception
    {
        variableService = new VariableServiceImpl(100, null, null, null);
        variableService.createNewVariable(null, "var1", "string", true, false, false, null, null);
        variableService.createNewVariable(null, "dummy", "string", true, false, false, null, null);
        variableService.createNewVariable(null, "intPrimitive", "int", true, false, false, null, null);
        varNode = new ExprVariableNodeImpl(variableService.getVariableMetaData("var1"), null);
    }

    public void testGetType()  throws Exception
    {
        SupportExprNodeFactory.validate3Stream(varNode);
        assertEquals(String.class, varNode.getConstantType());
    }

    public void testEvaluate() throws Exception
    {
        SupportExprNodeFactory.validate3Stream(varNode);
        assertEquals("my_variable_value", varNode.evaluate(null, true, null));
    }

    public void testEquals()  throws Exception
    {
        ExprInNode otherInNode = SupportExprNodeFactory.makeInSetNode(false);
        ExprVariableNodeImpl otherVarOne = new ExprVariableNodeImpl(variableService.getVariableMetaData("dummy"), null);
        ExprVariableNodeImpl otherVarTwo = new ExprVariableNodeImpl(variableService.getVariableMetaData("var1"), null);
        ExprVariableNodeImpl otherVarThree = new ExprVariableNodeImpl(variableService.getVariableMetaData("var1"), "abc");

        assertTrue(varNode.equalsNode(varNode));
        assertTrue(varNode.equalsNode(otherVarTwo));
        assertFalse(varNode.equalsNode(otherVarOne));
        assertFalse(varNode.equalsNode(otherInNode));
        assertFalse(otherVarTwo.equalsNode(otherVarThree));
    }

    public void testToExpressionString() throws Exception
    {
        assertEquals("var1", ExprNodeUtility.toExpressionStringMinPrecedence(varNode));
    }

    private void tryInvalidValidate(ExprVariableNodeImpl varNode) throws Exception
    {
        try {
            SupportExprNodeFactory.validate3Stream(varNode);
            fail();
        }
        catch (ExprValidationException ex)
        {
            // expected
        }
    }
}
